
#include "bsp.h"
#include "mrfi.h"
#include "nwk_types.h"
#include "nwk_api.h"
#include "nwk_frame.h"
#include "nwk.h"
#include "bsp_leds.h"
#include "bsp_buttons.h"

#include "app_remap_led.h"


//Prototypes
void toggleLED(uint8_t);
void startTimerA();
void resetTime();
void sendTouched();
void sendNonValid();
static uint8_t sCB(linkID_t);
void configurePorts();
static void linkTo(void);


static uint8_t state = 0;
static linkID_t sLinkID1 = 0;
static uint8_t seg = 0;
static uint8_t min = 0;
static uint8_t fencer;

static uint8_t capture = 0;

//INs
#define TIP BIT2
#define TIP2 BIT0
#define JACKET BIT1

//Tipe of touch
#define VALID 1
#define NONVALID 2

//States
#define JOINING 1
#define WAITING 2
#define TOUCHED 3
#define NONVALIDTOUCH 4


#define SPIN_ABOUT_A_SECOND   NWK_DELAY(1000)
#define SPIN_ABOUT_A_QUARTER_SECOND   NWK_DELAY(250)

/* How many times to try a Tx and miss an acknowledge before doing a scan */
#define MISSES_IN_A_ROW  2

void main (void)
{
  BSP_Init();
  
  WDTCTL = WDTPW|WDTHOLD;
  BSP_ENABLE_INTERRUPTS();


/* If an on-the-fly device address is generated it must be done before the
   * call to SMPL_Init(). If the address is set here the ROM value will not
   * be used. If SMPL_Init() runs before this IOCTL is used the IOCTL call
   * will not take effect. One shot only. The IOCTL call below is conformal.
   */
//#ifdef I_WANT_TO_CHANGE_DEFAULT_ROM_DEVICE_ADDRESS_PSEUDO_CODE
  //{
    addr_t lAddr={0x79,0x56,0x34,0x12};

    
    SMPL_Ioctl(IOCTL_OBJ_ADDR, IOCTL_ACT_SET, &lAddr);
//  }
//#endif /* I_WANT_TO_CHANGE_DEFAULT_ROM_DEVICE_ADDRESS_PSEUDO_CODE */





  /* Keep trying to join (a side effect of successful initialization) until
   * successful. Toggle LEDS to indicate that joining has not occurred.
   */
  
  state = JOINING; 
  
  //smplStatus_t rc2;
  
  while (SMPL_SUCCESS != SMPL_Init(sCB))
  {
    toggleLED(1);
    toggleLED(2);
    SPIN_ABOUT_A_SECOND;
  }

  /* LEDs on solid to indicate successful join. */
  if (!BSP_LED2_IS_ON()) toggleLED(2);
  if (!BSP_LED1_IS_ON()) toggleLED(1);
  

  /* Unconditional link to AP which is listening due to successful join. */
  linkTo();

  while (1); //Unreachable statement
}

static void linkTo()
{
  /*Preparing the end device to listen the AP too*/
  SMPL_Ioctl( IOCTL_OBJ_RADIO, IOCTL_ACT_RADIO_RXON, 0);

  /* Keep trying to link... */
  while (SMPL_SUCCESS != SMPL_Link(&sLinkID1))
  {
    toggleLED(1);
    toggleLED(2);
    SPIN_ABOUT_A_SECOND;
  }

  /* Turn off LEDs. */
  if (BSP_LED2_IS_ON()) toggleLED(2);
  
  if (BSP_LED1_IS_ON()) toggleLED(1);
 
  

  

  while (1)
  {
  
  	configurePorts();
  	
  	for(;;){
  		__low_power_mode_1();
  	}
  }
}

void sendTouched(){
	  
	  uint8_t      done, misses;
      //smplStatus_t rc;
      uint8_t     msg[1];

      /* Building the message to send */
      /*msg[4] = TAR;
      msg[3] = seg;
      msg[2] = min;
      msg[1] = fencer;*/
      msg[0] = VALID;
      done = 0;
      while (!done)
      {

        /* Try sending message MISSES_IN_A_ROW times */
        for (misses=0; misses < MISSES_IN_A_ROW; ++misses)
        {
          if (SMPL_SUCCESS == SMPL_Send(sLinkID1, msg, sizeof(msg)))
          {
            /* Message sent. We're done. Toggle LED 1. */
            //toggleLED(1);
            done=1;
            break;
          }
          
        }
        
	  }
}

void sendNonValid(){
	
	  uint8_t      done, misses;
      //smplStatus_t rc;
      uint8_t     msg[1];

      

      /* Building the message to send */
      /*msg[4] = 0;
      msg[3] = 0;
      msg[2] = 0;
      msg[1] = fencer;*/
      msg[0] = NONVALID;
      done = 0;
      while (!done)
      {
        

        /* Try sending message MISSES_IN_A_ROW times */
        for (misses=0; misses < MISSES_IN_A_ROW; ++misses)
        {
          if (SMPL_SUCCESS == SMPL_Send(sLinkID1, msg, sizeof(msg)))
          {
            /* Message sent. We're done. Toggle LED 1. */
            //toggleLED(1);
            startTimerA();
            
            
            done=1;
            break;
          }
          
        }
        
	  }
}



void toggleLED(uint8_t which)
{
  if (1 == which)
  {
    BSP_TOGGLE_LED1();
  }
  else if (2 == which)
  {
    BSP_TOGGLE_LED2();
  }
  return;
}


static uint8_t sCB(linkID_t lid)
{
	//uint8_t     msg[MAX_APP_PAYLOAD];
	//uint8_t		len;
	
	if (state == JOINING){
		//if (SMPL_SUCCESS == SMPL_Receive(SMPL_LINKID_USER_UUD/*lid*/, msg, &len))
	    //{
	    	//fencer=msg[0];
	    	//resetTime();
	    	//startTimerA();
	    	state = WAITING;
	    	return 0;
	    //}
	}
	
	else if (state == WAITING){
	
		//resetTime();
		
		return 0;
	
	}
	
	else if (state == TOUCHED || state == NONVALIDTOUCH){
		capture=TAR;
		state=WAITING;
		capture=0;
		resetTime();
		return 0;
	}
	
	
	else return 0;
}


void startTimerA(){
	/*
	TACCR0 = 49999; // Upper limit of count for TAR
	TACCTL0 = CCIE; // Enable interrupts on Compare 0
	TACTL = MC_3|ID_3|TASSEL_2|TACLR; // Set up and start Timer A
	// "Up to CCR0" mode , divide clock by 8, clock from SMCLK , clear timer
	//__enable_interrupt();
	*/
	
	/*
	
	BCSCTL3 |= LFXT1S_2;                      // LFXT1 = VLO
  	TACCTL0 = CCIE;                           // TACCR0 interrupt enabled
  	TACCR0 = 12000;                           // ~ 1 sec
  	TACTL = TASSEL_1 | MC_1 | TACLR;                  // ACLK, upmode
  	*/
}

void resetTime(){
	
	if (BSP_LED2_IS_ON()) toggleLED(2);
  
  	if (BSP_LED1_IS_ON()) toggleLED(1);
  	
	min=0;
	seg=0;
	startTimerA();
}

void configurePorts(){
	//P1DIR = BIT2
  	P1IE = TIP;
  	P1IES = TIP;
  	P2DIR &= ~JACKET;
  	P2DIR &= ~TIP2;
  	P2REN |= JACKET | TIP2;
  	P2OUT |= JACKET | TIP2;
  	P2IE |= JACKET;
  	P2IES |= JACKET;
}

#pragma vector = TIMERA0_VECTOR
__interrupt void TIMERA0_ISR (void) // ISR for TACCR1 CCIFG and TAIFG
{
	toggleLED(2);
	seg++;
	if(seg==60){
		
		min++;
		seg=0;
		
	}
			
}
	
#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR (void)
{
	//SMPL_Ioctl( IOCTL_OBJ_RADIO, IOCTL_ACT_RADIO_AWAKE, 0);
	
	if(!(P1IN & TIP) && state==WAITING){
	
		SPIN_ABOUT_A_QUARTER_SECOND;
		
		if((P2IN & TIP2)){
			sendNonValid();
			state=NONVALIDTOUCH;
		}
		/*else{
			P2IFG &= ~TIP2;
		}*/
		
		//P1IFG = 0;
	}
	
	
		
	do {
		P1IFG = 0; // Clear any pending interrupts ...
	} while (P1IFG != 0);
	
	//SMPL_Ioctl( IOCTL_OBJ_RADIO, IOCTL_ACT_RADIO_SLEEP, 0);
}



//#pragma vector = PORT2_VECTOR
//__interrupt void PORT2_ISR (void)
BSP_ISR_FUNCTION( BSP_GpioPort1Isr, PORT2_VECTOR )
{
	
	
	if(P2IFG & JACKET){
  	//He sido tocado valido.
  	
  		SPIN_ABOUT_A_QUARTER_SECOND;
  		if(state==WAITING || state==NONVALIDTOUCH) {
  			//SMPL_Ioctl( IOCTL_OBJ_RADIO, IOCTL_ACT_RADIO_AWAKE, 0);
  			sendTouched();
  			//SMPL_Ioctl( IOCTL_OBJ_RADIO, IOCTL_ACT_RADIO_SLEEP, 0);
			state=TOUCHED;
  		}
		
		P2IFG &= ~JACKET;
	}
		
   
	MRFI_GpioIsr();
	
	
	
}

